import { Component, EventEmitter, OnInit, Output } from '@angular/core';
import { ReviewManagementService } from 'app/admin/review-management/review-management.service';
import { Review, ReviewComment } from 'app/admin/review-management/review.model';
import { ReviewRequest } from 'app/admin/review-management/reviewRequest.model';
import { UserManagementService } from 'app/admin/user-management/service/user-management.service';
import { User } from 'app/admin/user-management/user-management.model';
import { Account } from 'app/core/auth/account.model';
import { AccountService } from 'app/core/auth/account.service';
import { ExerciseService } from 'app/exercise/service/exercise.service';
import { SearchService } from 'app/search/service/search-service';
import { ExtendedSearchResultDTO, SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';
import { SearchResultsDTO } from 'app/shared/model/search/search-results-dto.model';
import { jsPDF } from 'jspdf';

@Component({
  selector: 'jhi-peer-reviewing',
  templateUrl: './peer-reviewing.component.html',
  styleUrls: ['./peer-reviewing.component.scss'],
})
export class PeerReviewingComponent implements OnInit {
  isChecked = false;
  currentAccount: Account | null = null;
  user: User | null = null;
  results: SearchResultDTO[] = [];
  allExercises: SearchResultDTO[] = [];
  selectedExercise: SearchResultDTO | undefined;
  reviews: Review[] = [];
  otherReviews: Review[] = [];
  userReviews: Review[] = [];
  selectedReview = new Review(
    '',
    [''],
    [''],
    [new ReviewComment(false, false, '', false, false, '', false, false, '', false, '', false, false, '')]
  );
  statusOptions = [
    { value: 'REVIEW_IN_PROGRESS', translationKey: 'review.create.reviewInProgress' },
    { value: 'IMPROVEMENTS_REQUESTED', translationKey: 'review.create.improvementsRequested' },
    { value: 'REVIEW_REJECTED', translationKey: 'review.create.reviewRejected' },
    { value: 'REVIEW_COMPLETED', translationKey: 'review.create.reviewCompleted' },
  ];
  selectedResult?: SearchResultDTO;
  selectedStatus = '';
  showOtherHistory: boolean[] = [];
  showHistory: boolean[] = [];

  @Output() exerciseSelectionEvent = new EventEmitter<SearchResultDTO>();

  constructor(
    private userService: UserManagementService,
    private accountService: AccountService,
    private searchService: SearchService,
    private reviewManagementService: ReviewManagementService,
    private exerciseService: ExerciseService
  ) {}

  ngOnInit(): void {
    this.accountService.identity().subscribe(account => {
      this.currentAccount = account;
      this.getUserFromLogin();
    });
  }

  getUserFromLogin(): void {
    if (this.currentAccount != null && this.currentAccount.login != null) {
      this.userService.getLoggedInUser().subscribe(val => {
        this.user = val;
        this.searchForExercises();
        this.loadAll();
      });
    }
  }

  loadAll(): void {
    this.reviewManagementService.getAllByUser().subscribe(reviews => {
      this.reviews = reviews;
      if (this.user && this.user.email) {
        this.userReviews = reviews.filter(review => review.requestedBy == this.user!.email);
        this.showHistory = this.userReviews?.length ? Array.from(Array(this.userReviews.length).keys()).map(() => false) : [];
        this.otherReviews = reviews.filter(review => review.requestedBy != this.user!.email);
        this.showOtherHistory = this.otherReviews?.length ? Array.from(Array(this.otherReviews.length).keys()).map(() => false) : [];
      }
    });
  }

  setReviewerStatusForLogin(): void {
    if (this.user != null && this.currentAccount != null && this.user.login == this.currentAccount.login) {
      this.userService.updateReviewerStatus(!this.user.reviewingEnabled).subscribe();
    }
  }

  searchForExercises(): void {
    this.searchService.getResultsForLoggedInUser().subscribe({
      next: (data: SearchResultsDTO) => {
        this.results = this.results
          .concat(data.searchResult)
          .filter((value, index, self) => self.findIndex(t => t.metadata.title === value.metadata.title) === index);
      },
      error: () => alert('Search failed'),
    });
    this.searchService.getAllResources().subscribe({
      next: (data: SearchResultDTO[]) => {
        this.allExercises = this.allExercises
          .concat(data)
          .filter((value, index, self) => self.findIndex(t => t.metadata.title === value.metadata.title) === index);
      },
    });
  }

  shareExercise(exercise: SearchResultDTO) {
    this.selectedExercise = exercise;
  }

  selectExercise(gitlabURL: string): ExtendedSearchResultDTO {
    const exercise: ExtendedSearchResultDTO = this.exerciseService.populateExerciseWithData(
      this.allExercises.find(exercise => exercise.project.url == gitlabURL)!
    );
    return exercise;
  }

  requestReview(): void {
    if (this.currentAccount) {
      this.searchService
        .requestReview(
          new ReviewRequest(
            this.selectedExercise!.metadata.title,
            this.selectedExercise!.exerciseId,
            [this.currentAccount.email],
            this.selectedExercise!.project.url
          )
        )
        .subscribe({
          next: () => {
            // This function runs when the request is successful.
            this.loadAll();
          },
          error: () => {
            // This function runs when the request fails.
            alert('It seems like a Review for this Exercise already exists.'); // You can customize this message.
          },
        });
    }
  }

  resourceImproved(review: Review): void {
    review.status![0] = 'REVIEW_IMPROVED';
    this.reviewManagementService.notifyReviewer(review).subscribe(() => this.loadAll());
  }

  updateReview(): void {
    this.selectedReview.status![0] = this.selectedStatus;
    this.reviewManagementService.updateByUser(this.selectedReview).subscribe(() => this.loadAll());
  }

  deleteReview(review: Review) {
    review.status![0] = 'DELETION_REQUESTED';
    this.reviewManagementService.updateByUser(review).subscribe(() => this.loadAll());
  }

  toggleHistory(index: number) {
    this.showHistory[index] = !this.showHistory[index];
  }

  isHistoryVisible(index: number): boolean {
    return this.showHistory[index];
  }

  toggleOtherHistory(index: number) {
    this.showOtherHistory[index] = !this.showOtherHistory[index];
  }

  isOtherHistoryVisible(index: number): boolean {
    return this.showOtherHistory[index];
  }

  downloadPDF() {
    const doc = new jsPDF();
    let y = 10;

    this.selectedReview.comments!.forEach((comment, i) => {
      if (y > 280) {
        doc.addPage();
        y = 10;
      }

      doc.text(`User: ${this.selectedReview.users![i]}`, 10, y);
      y += 10;

      if (y + 40 > 280) {
        doc.addPage();
        y = 10;
      }
      doc.text(`Clarity and Understandability:`, 10, y);
      doc.text(`Description is clear: ${String(comment.descriptionIsClear)}`, 20, y + 10);
      doc.text(`Requirements are clear: ${String(comment.requirementsAreClear)}`, 20, y + 20);
      doc.text(`Comment: ${String(comment.descriptionComment)}`, 20, y + 30);
      y += 40;

      if (y + 30 > 280) {
        doc.addPage();
        y = 10;
      }
      doc.text(`Completeness:`, 10, y);
      doc.text(`Information required is complete: ${String(comment.informationRequiredComplete)}`, 20, y + 10);
      doc.text(`All materials are available: ${String(comment.allMaterialsAvailable)}`, 20, y + 20);
      y += 30;

      if (y + 40 > 280) {
        doc.addPage();
        y = 10;
      }
      doc.text(`Format and Structure:`, 10, y);
      doc.text(`Is structured: ${String(comment.isStructured)}`, 20, y + 10);
      doc.text(`Sub-exercises are clear: ${String(comment.subExercisesAreClear)}`, 20, y + 20);
      doc.text(`Comment: ${String(comment.structuredComment)}`, 20, y + 30);
      y += 40;

      if (y + 30 > 280) {
        doc.addPage();
        y = 10;
      }
      doc.text(`Validity and Correctness:`, 10, y);
      doc.text(`Solution is correct: ${String(comment.solutionIsCorrect)}`, 20, y + 10);
      doc.text(`Comment: ${String(comment.solutionComment)}`, 20, y + 20);
      y += 30;

      if (y + 40 > 280) {
        doc.addPage();
        y = 10;
      }
      doc.text(`Requirements and Metadata:`, 10, y);
      doc.text(`Metadata is complete: ${String(comment.metadataIsComplete)}`, 20, y + 10);
      doc.text(`Metadata is correct: ${String(comment.metadataIsCorrect)}`, 20, y + 20);
      doc.text(`Comment: ${String(comment.metadataComment)}`, 20, y + 30);
      y += 40;
    });

    doc.save('comments.pdf');
  }
}
