import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IWatchListEntry, WatchListEntry } from '../watch-list-entry.model';

import { WatchListEntryService } from './watch-list-entry.service';

describe('WatchListEntry Service', () => {
  let service: WatchListEntryService;
  let httpMock: HttpTestingController;
  let elemDefault: IWatchListEntry;
  let expectedResult: IWatchListEntry | IWatchListEntry[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(WatchListEntryService);
    httpMock = TestBed.inject(HttpTestingController);

    elemDefault = {
      id: 0,
      exerciseId: 'AAAAAAA',
      exerciseName: 'AAAAAAA',
    };
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = Object.assign({}, elemDefault);

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(elemDefault);
    });

    it('should create a WatchListEntry', () => {
      const returnedFromService = Object.assign(
        {
          id: 0,
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.create(new WatchListEntry()).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a WatchListEntry', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          exerciseId: 'BBBBBB',
          exerciseName: 'BBBBBB',
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.update(expected).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a WatchListEntry', () => {
      const patchObject = Object.assign(
        {
          exerciseId: 'BBBBBB',
        },
        new WatchListEntry()
      );

      const returnedFromService = Object.assign(patchObject, elemDefault);

      const expected = Object.assign({}, returnedFromService);

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of WatchListEntry', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          exerciseId: 'BBBBBB',
          exerciseName: 'BBBBBB',
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toContainEqual(expected);
    });

    it('should delete a WatchListEntry', () => {
      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult);
    });

    describe('addWatchListEntryToCollectionIfMissing', () => {
      it('should add a WatchListEntry to an empty array', () => {
        const watchListEntry: IWatchListEntry = { id: 123 };
        expectedResult = service.addWatchListEntryToCollectionIfMissing([], watchListEntry);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(watchListEntry);
      });

      it('should not add a WatchListEntry to an array that contains it', () => {
        const watchListEntry: IWatchListEntry = { id: 123 };
        const watchListEntryCollection: IWatchListEntry[] = [
          {
            ...watchListEntry,
          },
          { id: 456 },
        ];
        expectedResult = service.addWatchListEntryToCollectionIfMissing(watchListEntryCollection, watchListEntry);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a WatchListEntry to an array that doesn't contain it", () => {
        const watchListEntry: IWatchListEntry = { id: 123 };
        const watchListEntryCollection: IWatchListEntry[] = [{ id: 456 }];
        expectedResult = service.addWatchListEntryToCollectionIfMissing(watchListEntryCollection, watchListEntry);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(watchListEntry);
      });

      it('should add only unique WatchListEntry to an array', () => {
        const watchListEntryArray: IWatchListEntry[] = [{ id: 123 }, { id: 456 }, { id: 68721 }];
        const watchListEntryCollection: IWatchListEntry[] = [{ id: 123 }];
        expectedResult = service.addWatchListEntryToCollectionIfMissing(watchListEntryCollection, ...watchListEntryArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const watchListEntry: IWatchListEntry = { id: 123 };
        const watchListEntry2: IWatchListEntry = { id: 456 };
        expectedResult = service.addWatchListEntryToCollectionIfMissing([], watchListEntry, watchListEntry2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(watchListEntry);
        expect(expectedResult).toContain(watchListEntry2);
      });

      it('should accept null and undefined values', () => {
        const watchListEntry: IWatchListEntry = { id: 123 };
        expectedResult = service.addWatchListEntryToCollectionIfMissing([], null, watchListEntry, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(watchListEntry);
      });

      it('should return initial array if no WatchListEntry is added', () => {
        const watchListEntryCollection: IWatchListEntry[] = [{ id: 123 }];
        expectedResult = service.addWatchListEntryToCollectionIfMissing(watchListEntryCollection, undefined, null);
        expect(expectedResult).toEqual(watchListEntryCollection);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
