import { Injectable, OnDestroy } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Observable, Subscription } from 'rxjs';
import { UserWatchListService } from 'app/entities/user-watch-list/user-watch-list.service';
import { WatchListEntryService } from 'app/entities/watch-list-entry/watch-list-entry.service';
import { TranslateService } from '@ngx-translate/core';
import { IUserWatchList, UserWatchListWithContent } from 'app/shared/model/user-watch-list.model';
import { CheckFrequency } from 'app/shared/model/enumerations/check-frequency.model';
import { IWatchListEntry } from 'app/shared/model/watch-list-entry.model';
import { AccountService } from 'app/core/auth/account.service';

import { Account } from 'app/core/auth/account.model';
import { share } from 'rxjs/operators';
import { SearchResultDTO } from '../model/search/search-result-dto.model';

type EntityResponseType = HttpResponse<IUserWatchList>;

/**
 provides infrastructure for watchlists management.
 The manager is is charge to guarantee that the local watchlists and the watchlists on the server are in synchrony.
 If the user is not logged in, the watchlists are undefined.
 */
@Injectable({ providedIn: 'root' })
export class WatchlistManager implements OnDestroy {
  account: Account | null = null;
  authSubscription?: Subscription;

  myWatchLists?: IUserWatchList[];
  currentWatchlist?: UserWatchListWithContent;

  /** flag, iff default bookmark list is about to be created */
  defaultBookmarkListInProgress = false;

  constructor(
    protected watchListService: UserWatchListService,
    protected watchListEntryService: WatchListEntryService,
    private accountService: AccountService,
    private translate: TranslateService
  ) {
    this.authSubscription = this.accountService.getAuthenticationState().subscribe(account => {
      this.account = account;
      if (this.accountService.isAuthenticated()) {
        this.watchListService
          .findMyWatchlists({
            page: 0,
            size: 50, // should be enough
            sort: '', // no sorting
          })
          .subscribe((data: HttpResponse<IUserWatchList[]>) => {
            if (data.body) {
              this.myWatchLists = data.body;
              if (this.myWatchLists.length > 0) {
                this.loadCurrentWatchList(this.myWatchLists[0]);
              } else {
                if (!this.defaultBookmarkListInProgress) {
                  //  if watchlist is empty, define default watchlist :-)
                  this.defaultBookmarkListInProgress = true;

                  const userWatchlist: IUserWatchList = {
                    name: this.translate.instant('gitsearchApp.userWatchList.defaultName'),
                    userLogin: this.account!.login,
                    userId: 1, // unused, replaced by correct userId by create service
                    checkFrequency: CheckFrequency.NEVER,
                  };
                  this.watchListService.createForCurrentUser(userWatchlist).subscribe({
                    next: (wl: HttpResponse<IUserWatchList>) => {
                      this.myWatchLists = [wl.body!];
                      this.loadCurrentWatchList(wl.body!);
                      this.defaultBookmarkListInProgress = false;
                    },
                    error: () => {
                      // error case
                      this.defaultBookmarkListInProgress = false;
                    },
                  });

                  this.currentWatchlist = undefined;
                }
              }
            }
          });
      } else {
        this.myWatchLists = undefined;
        this.currentWatchlist = undefined;
      }
    });
  }

  public isExerciseOnCurrentWatchlist(e: SearchResultDTO): boolean {
    if (this.currentWatchlist) {
      const elem = this.currentWatchlist.exercises.find(entry => entry.exerciseId === e.exerciseId);
      return elem !== undefined;
    }
    return false;
  }

  private loadCurrentWatchList(wl: IUserWatchList): void {
    this.watchListEntryService.getWatchListContent(wl.id!).subscribe((wles: HttpResponse<IWatchListEntry[]>) => {
      this.currentWatchlist = {
        userWatchList: wl,
        exercises: wles.body!,
      };
    });
  }

  public getMyWatchLists(): IUserWatchList[] | undefined {
    return this.myWatchLists;
  }

  public getCurrentWatchList(): UserWatchListWithContent | undefined {
    return this.currentWatchlist;
  }

  public handleCheckForCurrentWatchlist(e: SearchResultDTO): Promise<void> {
    return new Promise<void>((resolve, reject) => {
      let handling;

      if (this.currentWatchlist) {
        if (!this.isExerciseOnCurrentWatchlist(e)) {
          // add
          const watchListEntry: IWatchListEntry = {
            exerciseId: e.exerciseId,
            exerciseName: e.metadata.title,
            watchlistId: this.currentWatchlist.userWatchList.id,
          };
          // reload watchlist
          handling = this.watchListEntryService.createForCurrentUser(watchListEntry);
        } else {
          // delete
          handling = this.watchListEntryService.deleteForCurrentUser(this.currentWatchlist.userWatchList.id!, e.exerciseId);
        }
        handling.subscribe({
          next: () => {
            if (this.currentWatchlist) this.loadCurrentWatchList(this.currentWatchlist.userWatchList);
            resolve();
          },
          error: err => {
            reject(err);
          },
        });
      }
    });
  }

  public createForCurrentUser(userWatchList: IUserWatchList): Observable<EntityResponseType> {
    const createCall = this.watchListService.createForCurrentUser(userWatchList).pipe(share());
    createCall.subscribe(createdWatchList => {
      this.myWatchLists!.push(createdWatchList.body!);
    });

    return createCall;
  }

  public update(userWatchList: IUserWatchList): Observable<EntityResponseType> {
    const updateCall = this.watchListService.update(userWatchList).pipe(share());
    updateCall.subscribe(createdWatchList => {
      const id = createdWatchList.body!.id!;
      if (this.myWatchLists) {
        for (let i = 0; i < this.myWatchLists.length; i++)
          if (this.myWatchLists[i].id === id) {
            this.myWatchLists[i] = createdWatchList.body!;
          }
      }
    });
    return updateCall;
  }

  public setCurrentWatchList(watchListName: string): void {
    if (this.myWatchLists) {
      this.myWatchLists.forEach(watchList => {
        if (watchList.name === watchListName) {
          this.loadCurrentWatchList(watchList);
        }
      });
    }
  }

  ngOnDestroy(): void {
    if (this.authSubscription) {
      this.authSubscription.unsubscribe();
    }
  }
}
